using System.Data;
using System.DirectoryServices.ActiveDirectory;
using System.Globalization;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;
using Microsoft.IdentityModel.Tokens;
using System.Data.SQLite;

namespace Medventory
{
    public partial class Form1_Main : Form
    {
        public static class Datenbank_Konfiguration
        {
            public static string Server = @"Medventory_Lokal.db";
            public static string Database = "Medventory";
            public static string User_Id = "medventory_admin";
            public static string Password = "medventory";
            public static string Table = "Medventory_Table";
        }

        public Form1_Main()
        {
            CultureInfo culture = new CultureInfo("en-US");
            Thread.CurrentThread.CurrentCulture = culture;
            Thread.CurrentThread.CurrentUICulture = culture;

            InitializeComponent();
        }

        public void Datenbank_WarnungenCheck(DataGridView Daten)
        {
            int Warnungen = 0;
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;"; 
            
            string SQL_Query = $"SELECT COUNT(*) FROM {Datenbank_Konfiguration.Table} WHERE Menge < Minimum";

            using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
            {
                try
                {
                    connection.Open();

                    using (SQLiteCommand command = new SQLiteCommand(SQL_Query, connection))
                    {
                        var result = command.ExecuteScalar();

                        if (result != null && result != DBNull.Value)
                        {
                            Warnungen = Convert.ToInt32(result);
                        }
                    }
                }
                catch (SQLiteException Fehler_SQL)
                {
                    MessageBox.Show("SQL Fehler beim Abrufen der Zeilenanzahl: " + Fehler_SQL.Message, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception)
                {
                    MessageBox.Show("Unbekannter Fehler beim Abrufen der Zeilenanzahl.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            //Wenn Warnungen aufgetreten sind...
            if (Warnungen > 0)
            {
                //Button zur Warnungs-Anzeige aktivieren
                Button_Warnungen.Enabled = true;
                Button_Warnungen.BackColor = Color.DarkRed;
                if (Warnungen == 1) //Wenn nur 1 Warnung aufgetreten ist
                    Button_Warnungen.Text = "1 Warnung";
                else //Wenn mehr als 1 Warnung aufgetreten ist
                    Button_Warnungen.Text = Warnungen + " Warnungen";
            }
            else //Keine Warnungen aufgetreten...
            {
                //Button zur Warnungs-Anzeige deaktivieren
                Button_Warnungen.Enabled = false;
                Button_Warnungen.BackColor = Color.Gray;
                Button_Warnungen.Text = "0 Warnungen";
            }
        }

        public void Datenbank_SpeichernAenderungen(DataGridView Daten_Aenderungen)
        {
            // SQL Server - Verbindung
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            // Variable to track if any changes were detected
            bool Aenderungen_Gefunden = false;
            string Aenderungen_Info = "";

            // Loop durch DataGridView
            foreach (DataGridViewRow row in Daten_Aenderungen.Rows)
            {
                // ID aktueller Zeile ermitteln
                var rowId = row.Cells[0].Value;

                // Werte aus Datenbank abrufen
                string SQL_Query = $"SELECT * FROM {Datenbank_Konfiguration.Table} WHERE ID = @ID";
                using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
                {
                    try
                    {
                        connection.Open();
                        SQLiteCommand fetchCommand = new SQLiteCommand(SQL_Query, connection);
                        fetchCommand.Parameters.AddWithValue("@ID", rowId);

                        SQLiteDataReader reader = fetchCommand.ExecuteReader();

                        if (reader.Read())
                        {
                            // Start building the changes summary for this row
                            string Zeile_Aenderungen = $"nderungen in Zeile mit ID: {rowId}\n";
                            bool Zeile_Geaendert = false;

                            // Zellen vergleichen mit Datenbank
                            for (int i = 1; i < row.Cells.Count; i++)
                            {
                                string columnName = Daten_Aenderungen.Columns[i].HeaderText;
                                var Wert_Aktuell = row.Cells[i].Value;
                                var Wert_Original = reader[columnName]; //Datenbank Wert abrufen

                                // Wenn der Wert sich gendert hat...
                                if (!object.Equals(Wert_Original, Wert_Aktuell))
                                {
                                    Zeile_Aenderungen += $"{columnName}: {Wert_Original} -> {Wert_Aktuell}\n";
                                    Zeile_Geaendert = true;
                                    Aenderungen_Gefunden = true;
                                }
                            }

                            // If any change was detected in the current row, add it to the summary
                            if (Zeile_Geaendert)
                            {
                                Aenderungen_Info += Zeile_Aenderungen + "\n";
                            }
                        }
                        reader.Close();
                    }
                    catch (SQLiteException ex)
                    {
                        MessageBox.Show("Dieser Datensatz kann nicht bearbeitet werden!\n\nFehlermeldung:\n" + ex.Message, "Datenbank Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                }
            }

            // Wenn keine nderungen gefunden wurden...
            if (!Aenderungen_Gefunden)
            {
                MessageBox.Show("Es wurden keine Werte gendert.", "Keine nderungen gefunden", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            // Dialogbox - Sollen die Werte gespeichert werden?
            DialogResult InfoDialog = MessageBox.Show(
                "Die folgenden nderungen wurden gemacht:\n\n" + Aenderungen_Info + "\nSollen die nderungen gespeichert werden?",
                "nderungen speichern",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Question
            );

            // JA
            if (InfoDialog == DialogResult.Yes)
            {
                using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
                {
                    try
                    {
                        connection.Open();

                        // Loop durch Zeilen
                        foreach (DataGridViewRow row in Daten_Aenderungen.Rows)
                        {
                            var rowId = row.Cells[0].Value;
                            string SQL_Query_Speichern = $"UPDATE {Datenbank_Konfiguration.Table} SET ";

                            //Zeilen Zuordnungen speichern
                            List<string> Zeilen_Zuordnungen = new List<string>();

                            // Loop durch Zellen
                            for (int i = 1; i < row.Cells.Count; i++)
                            {
                                var Wert_Aktuell = row.Cells[i].Value;
                                var Wert_Original = Daten_Aenderungen.Rows[row.Index].Cells[i].Tag;

                                // Wenn der Wert sich gendert hat -> zur SQL Query hinzufgen
                                if (!object.Equals(Wert_Original, Wert_Aktuell))
                                {
                                    string Spalte = Daten_Aenderungen.Columns[i].HeaderText;
                                    Zeilen_Zuordnungen.Add($"{Spalte} = '{Wert_Aktuell}'");
                                }
                            }

                            // Wenn nderungen in der Zeile vorhanden sind
                            if (Zeilen_Zuordnungen.Count > 0)
                            {
                                SQL_Query_Speichern += string.Join(", ", Zeilen_Zuordnungen) + " WHERE ID = '" + rowId + "'";

                                // Execute the update query for this row
                                SQLiteCommand command = new SQLiteCommand(SQL_Query_Speichern, connection);
                                command.ExecuteNonQuery();
                            }
                        }

                        MessageBox.Show("nderungen wurden gespeichert.", "Speichern", MessageBoxButtons.OK, MessageBoxIcon.Information);

                        //Tabelle aktualisieren
                        Datenbank_Refresh();
                    }
                    catch (SQLiteException Fehler_SQL)
                    {
                        MessageBox.Show("SQL Fehler beim Speichern der nderungen: " + Fehler_SQL.Message, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Unbekannter Fehler beim Speichern der nderungen: " + ex.Message, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void Datenbank_Loeschen(DataGridView Daten_Aenderungen)
        {
            // SQL Server - Verbindung
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            // SQL Query
            string SQL_Query = $"DELETE FROM {Datenbank_Konfiguration.Table} WHERE ID = '" + Daten_Aenderungen.CurrentRow.Cells[0].Value + "'";

            using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
            {
                try
                {
                    connection.Open();

                    SQLiteCommand command = new SQLiteCommand(SQL_Query, connection);

                    //Loeschung
                    command.ExecuteNonQuery();
                }
                catch (SQLiteException Fehler_SQL)
                {
                    MessageBox.Show("Dieser Datensatz kann nicht gelscht werden!\n\nFehlermeldung:\n" + Fehler_SQL.Message, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception)
                {
                    MessageBox.Show("Unbekannter Fehler beim Loeschen.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        public void Datenbank_LoeschenDialog(DataGridView Daten_Aenderungen)
        {
            //Wenn eine Zeile ausgewhlt wurde...
            if (Daten_Aenderungen.CurrentCell != null)
            {
                //Zeilen-Auswahl speichern
                DataGridViewRow Zeile_Auswahl = Daten_Aenderungen.Rows[Daten_Aenderungen.CurrentCell.RowIndex];

                //Info - Erstellen
                string Info_Loeschung = "Die folgende Zeile wird gelscht:\n";

                foreach (DataGridViewCell cell in Zeile_Auswahl.Cells)
                {
                    Info_Loeschung += $"{Daten_Aenderungen.Columns[cell.ColumnIndex].HeaderText}: {cell.Value}\n";
                }

                //Info - Ausgabe
                DialogResult Info_Ausgabe = MessageBox.Show
                    (
                    Info_Loeschung + "\nSoll diese Zeile gelscht werden?",
                    "LSCHEN",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question
                    );

                //Ja
                if (Info_Ausgabe == DialogResult.Yes)
                {
                    Datenbank_Loeschen(Daten_Aenderungen);

                    Datenbank_Refresh();
                }
                else //Nein
                {
                    //Keine Aktion
                }
            }
            else //Wenn keine Zeile ausgewhlt wurde...
            {
                MessageBox.Show("Es wurde keine Zeile markiert.", "Fehler: Keine Auswahl", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        public int Datenbank_ZeilenGesamt()
        {
            int ZeilenGesamt = 0;
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            string SQL_Query = $"SELECT COUNT(*) FROM {Datenbank_Konfiguration.Table}";

            using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
            {
                try
                {
                    connection.Open();

                    using (SQLiteCommand command = new SQLiteCommand(SQL_Query, connection))
                    {
                        var result = command.ExecuteScalar();
                        ZeilenGesamt = result != DBNull.Value ? Convert.ToInt32(result) : 0;
                    }
                }
                catch (SQLiteException Fehler_SQL)
                {
                    MessageBox.Show("SQL Fehler beim Abrufen der Zeilenanzahl: " + Fehler_SQL.Message, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception)
                {
                    MessageBox.Show("Unbekannter Fehler beim Abrufen der Zeilenanzahl.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            return ZeilenGesamt;
        }

        public bool FormatierungCheck(TextBox Artikel, ComboBox Art, TextBox Hersteller, NumericUpDown Menge, NumericUpDown Minimum, NumericUpDown Preis, TextBox Identifikationsnummer)
        {
            bool Fehler = false;

            //Artikel
            if (string.IsNullOrWhiteSpace(Artikel.Text))
            {
                //Ungltig
                Fehler = true;

                //Status
                MessageBox.Show("Der Artikel ist ungltig!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Art
            if (string.IsNullOrWhiteSpace(Art.Text) && Fehler == false)
            {
                //Ungltig
                Fehler = true;

                //Status
                MessageBox.Show("Die Art ist ungltig!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Hersteller
            if (string.IsNullOrWhiteSpace(Hersteller.Text) && Fehler == false)
            {
                //Ungltig
                Fehler = true;

                //Status
                MessageBox.Show("Der Hersteller ist ungltig!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Menge
            int Menge_Check;
            if (!int.TryParse(Menge.Text, out Menge_Check) && Fehler == false)
            {
                //Ungltig
                Fehler = true;

                //Status
                MessageBox.Show("Die Menge ist ungltig!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Minimum
            if ((Minimum.Value != (int)Minimum.Value) && Fehler == false)
            {
                //Ungltig
                Fehler = true;

                //Status
                MessageBox.Show("Das Minimum ist ungltig!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Preis
            double Preis_Check;
            if (!double.TryParse(Preis.Text, out Preis_Check) && Fehler == false)
            {
                //Ungltig
                Fehler = true;

                //Status
                MessageBox.Show("Der Preis ist ungltig!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Identifikationsnummer
            if (string.IsNullOrWhiteSpace(Identifikationsnummer.Text) && Fehler == false)
            {
                //Ungltig
                Fehler = true;

                //Status
                MessageBox.Show("Die Identifikationsnummer ist ungltig!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return Fehler;
        }

        public void Datenbank_Suche(string Suche_Artikel, string Suche_Art, string Suche_Hersteller, string Suche_Menge, string Suche_Minimum, string Suche_Preis, string Suche_Identifikationsnummer, DateTime Suche_Von, DateTime Suche_Bis)
        {
            // SQL Server - Verbindung
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            // SQL Query
            string SQL_Query = $"SELECT * FROM {Datenbank_Konfiguration.Table} WHERE " +
                               "(('" + Suche_Artikel + "' = '' OR Artikel LIKE '%" + Suche_Artikel + "%') AND " +
                               "('" + Suche_Art + "' = '' OR Art LIKE '%" + Suche_Art + "%') AND " +
                               "('" + Suche_Hersteller + "' = '' OR Hersteller LIKE '%" + Suche_Hersteller + "%') AND " +
                               "('" + Suche_Menge + "' = '' OR Menge = " + (string.IsNullOrEmpty(Suche_Menge) ? "NULL" : Convert.ToInt32(Suche_Menge)) + ") AND " +
                               "('" + Suche_Minimum + "' = '' OR Minimum = " + (string.IsNullOrEmpty(Suche_Minimum) ? "NULL" : Convert.ToInt32(Suche_Minimum)) + ") AND " +
                               "('" + Suche_Preis + "' = '' OR Preis = " + (string.IsNullOrEmpty(Suche_Preis) ? "NULL" : Convert.ToDecimal(Suche_Preis)) + ") AND " +
                               "('" + Suche_Identifikationsnummer + "' = '' OR Identifikationsnummer LIKE '%" + Suche_Identifikationsnummer + "%') AND " +
                               "( Aktualisiert >= '" + Suche_Von.ToString("yyyy-MM-dd HH:mm:ss") + "' AND Aktualisiert <= '" + Suche_Bis.ToString("yyyy-MM-dd HH:mm:ss") + "')) " +
                               "ORDER BY Aktualisiert DESC";

            // Daten in Datentabelle bertragen
            try
            {
                //Verbindung
                using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
                {
                    connection.Open();

                    SQLiteDataAdapter adapter = new SQLiteDataAdapter(SQL_Query, connection);

                    DataTable dataTable = new DataTable();

                    adapter.Fill(dataTable);

                    Medventory_DataTable.DataSource = dataTable;
                }
            }
            catch (SQLiteException Fehler_SQL)
            {
                // FEHLER: SQL
                string Fehler_Text = "SQL Fehler: " + Fehler_SQL.Message;
                //"\nError Code: " + Fehler_SQL.ErrorCode +
                //"\nState: " + Fehler_SQL.State +
                //"\nStack Trace: " + Fehler_SQL.StackTrace;

                MessageBox.Show(Fehler_Text, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);

            }
            catch (Exception)
            {
                // FEHLER: Sonstiges
                MessageBox.Show("Unbekannter Fehler.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Anzeige Datensatz anpassen (Wenn Eintrge gefiltert wurden)
            if (Medventory_DataTable.RowCount < Datenbank_ZeilenGesamt())
            {
                Anzeige_Datensatz.Text = "Gefilterte Eintrge (" + Medventory_DataTable.RowCount.ToString() + ")";
                Anzeige_Datensatz.BackColor = Color.Navy;
            }
            else
            {
                Anzeige_Datensatz.Text = "Alle Eintrge (" + Medventory_DataTable.RowCount.ToString() + ")";
                Anzeige_Datensatz.BackColor = Color.SteelBlue;
            }

            //Button Farben anpassen
            Button_Kostenrechnung.BackColor = Color.Gray;
            if (Button_Warnungen.Text != "0 Warnungen")
                Button_Warnungen.BackColor = Color.DarkRed;
            else
                Button_Warnungen.BackColor = Color.Gray;

            //Bearbeitungs-Buttons aktivieren
            Button_Speichern.Enabled = true;
            Button_Loeschen.Enabled = true;

            //Datum Format anpassen
            Medventory_DataTable.Columns["Aktualisiert"].DefaultCellStyle.Format = "yyyy-MM-dd HH:mm:ss";
        }


        public void Datenbank_Refresh()
        {
            // SQL Server - Verbindung
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            // SQL Query
            string SQL_Query = $"SELECT * FROM {Datenbank_Konfiguration.Table} ORDER BY Aktualisiert DESC";

            // Daten in Datentabelle bertragen
            try
            {
                //Verbindung
                using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
                {
                    connection.Open();

                    SQLiteDataAdapter adapter = new SQLiteDataAdapter(SQL_Query, connection);

                    DataTable dataTable = new DataTable();

                    adapter.Fill(dataTable);

                    Medventory_DataTable.DataSource = dataTable;

                    //Spalten-Reihenfolge festlegen
                    Medventory_DataTable.AutoGenerateColumns = false;
                    Medventory_DataTable.Columns["ID"].DisplayIndex = 0;
                    Medventory_DataTable.Columns["Artikel"].DisplayIndex = 1;
                    Medventory_DataTable.Columns["Art"].DisplayIndex = 2;
                    Medventory_DataTable.Columns["Hersteller"].DisplayIndex = 3;
                    Medventory_DataTable.Columns["Menge"].DisplayIndex = 4;
                    Medventory_DataTable.Columns["Minimum"].DisplayIndex = 5;
                    Medventory_DataTable.Columns["Preis"].DisplayIndex = 6;
                    Medventory_DataTable.Columns["Identifikationsnummer"].DisplayIndex = 7;
                    Medventory_DataTable.Columns["Aktualisiert"].DisplayIndex = 8;
                    Medventory_DataTable.AutoGenerateColumns = true;
                }
            }
            catch (SQLiteException Fehler_SQL)
            {
                // FEHLER: SQL
                string Fehler_Text = "SQL Fehler: " + "Die Verbindung mit dem SQL-Server konnte nicht hergestellt werden.";
                string Fehler_Text_Komplett = "SQL Fehler: " + Fehler_SQL.Message;
                //"\nFehler Code: " + Fehler_SQL.ErrorCode;
                //"\nState: " + Fehler_SQL.State;
                //"\nStack Trace: " + Fehler_SQL.StackTrace;

                MessageBox.Show(Fehler_Text, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);

                //PROGRAMM-ABBRUCH
                Application.Exit();
                return;

            }
            catch (Exception)
            {
                // FEHLER: Sonstiges
                MessageBox.Show("Unbekannter Fehler.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Farben + Texte anpassen
            Anzeige_Datensatz.Text = "Alle Eintrge (" + Medventory_DataTable.RowCount.ToString() + ")";
            Anzeige_Datensatz.BackColor = Color.SteelBlue;
            Button_Kostenrechnung.BackColor = Color.Gray;
            if (Button_Warnungen.Text != "0 Warnungen")
                Button_Warnungen.BackColor = Color.DarkRed;
            else
                Button_Warnungen.BackColor = Color.Gray;

            //Datum BIS aktualisieren
            Suche_DatumBis.Value = DateTime.Now;

            //Warnungen prfen
            Datenbank_WarnungenCheck(Medventory_DataTable);

            //Bearbeitungs-Buttons aktivieren
            Button_Speichern.Enabled = true;
            Button_Loeschen.Enabled = true;

            //Datum Format anpassen
            Medventory_DataTable.Columns["Aktualisiert"].DefaultCellStyle.Format = "yyyy-MM-dd HH:mm:ss";
        }

        private void Datenbank_Eintrag(string Artikel, string Art, string Hersteller, int Menge, int Minimum, double Preis, string Identifikationsnummer, string Aktualisiert)
        {
            // SQL Server - Verbindung
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            // Aktuelles Datum + Zeit ermitteln
            string Zeitpunkt = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff");

            // SQL Query
            string SQL_Query = $"INSERT INTO {Datenbank_Konfiguration.Table} (Artikel, Art, Hersteller, Menge, Minimum, Preis, Identifikationsnummer, Aktualisiert) " +
                   "VALUES('" + Artikel + "', '" + Art + "', '" + Hersteller + "', " + Menge + ", " + Minimum + ", " + Preis + ", '" + Identifikationsnummer + "', '" + Aktualisiert + "')";

            // Daten in Datentabelle bertragen
            try
            {
                //Verbindung
                using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
                {
                    connection.Open();

                    SQLiteDataAdapter adapter = new SQLiteDataAdapter(SQL_Query, connection);

                    DataTable dataTable = new DataTable();

                    adapter.Fill(dataTable);

                    Medventory_DataTable.DataSource = dataTable;

                    MessageBox.Show("Neuer Eintrag wurde erfolgreich erstellt!", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (SQLiteException Fehler_SQL)
            {
                // FEHLER: SQL
                string Fehler_Text = "SQL Fehler: " + Fehler_SQL.Message;
                //"\nFehler Code: " + Fehler_SQL.ErrorCode;
                //"\nState: " + Fehler_SQL.State +
                //"\nStack Trace: " + Fehler_SQL.StackTrace;

                MessageBox.Show(Fehler_Text, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);

            }
            catch (Exception)
            {
                // FEHLER: Sonstiges
                MessageBox.Show("Unbekannter Fehler.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //Beim Start der Applikation die Datenstze aus der Datenbank laden
            Datenbank_Refresh();
        }

        private void DEBUG_Read_Click(object sender, EventArgs e)
        {
            Datenbank_Refresh();
        }

        private void DEBUG_Write_Click(object sender, EventArgs e)
        {
            Datenbank_Eintrag("Test1", "Test2", "Test3", 123, 1234, 1.19, "Test4", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff"));
        }

        private void button3_Click(object sender, EventArgs e)
        {

        }

        private void Anzeige_Alles_Click(object sender, EventArgs e)
        {

        }

        private void Anzeige_Alles_Click_1(object sender, EventArgs e)
        {

        }

        private void button3_Click_1(object sender, EventArgs e)
        {
            Datenbank_Refresh();
        }

        private void textBox3_TextChanged(object sender, EventArgs e)
        {

        }

        private void Button_NeuerEintrag_Click(object sender, EventArgs e)
        {
            //Formatierungsfehler prfen
            bool Fehler = FormatierungCheck(EintragNeu_Artikel, EintragNeu_Art, EintragNeu_Hersteller, EintragNeu_Menge, EintragNeu_Minimum, EintragNeu_Preis, EintragNeu_Identifikationsnummer);

            //Wenn keine Fehler aufetreten sind & Formatierung abgeschlossen ist...
            if (Fehler == false)
            {
                //Werte eintragen
                Datenbank_Eintrag(EintragNeu_Artikel.Text, EintragNeu_Art.Text, EintragNeu_Hersteller.Text, Convert.ToInt32(EintragNeu_Menge.Text), Convert.ToInt32(EintragNeu_Minimum.Value), Convert.ToDouble(EintragNeu_Preis.Text), EintragNeu_Identifikationsnummer.Text, DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));

                //Datenbank neu laden
                Datenbank_Refresh();

            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            Datenbank_Refresh();
        }

        private void Medventory_DataTable_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {

        }

        private void Anzeige_Alles_Click_2(object sender, EventArgs e)
        {
            Datenbank_Refresh();
        }

        private void Button_Suchen_Click(object sender, EventArgs e)
        {
            //Null-Formatierungs-Check
#nullable disable
            string Suche_Menge_Formatiert = null;
            string Suche_Minimum_Formatiert = null;
            string Suche_Preis_Formatiert = null;
            if (Suche_Menge.Enabled == true)
                Suche_Menge_Formatiert = Suche_Menge.Value.ToString();

            if (Suche_Minimum.Enabled == true)
                Suche_Minimum_Formatiert = Suche_Minimum.Value.ToString();

            if (Suche_Preis.Enabled == true)
                Suche_Preis_Formatiert = Suche_Preis.Value.ToString();

            //Suche
            Datenbank_Suche(Suche_Artikel.Text, Suche_Art.Text, Suche_Hersteller.Text, Suche_Menge_Formatiert, Suche_Minimum_Formatiert, Suche_Preis_Formatiert, Suche_Identifikationsnummer.Text, Suche_DatumVon.Value, Suche_DatumBis.Value);
#nullable enable
        }

        private void label9_Click(object sender, EventArgs e)
        {

        }

        private void Button_Speichern_Click(object sender, EventArgs e)
        {
            Datenbank_SpeichernAenderungen(Medventory_DataTable);
        }

        private void Medventory_DataTable_CellClick(object sender, DataGridViewCellEventArgs e)
        {

        }

        private void Medventory_DataTable_SelectionChanged(object sender, EventArgs e)
        {

        }

        private void Button_Loeschen_Click(object sender, EventArgs e)
        {
            Datenbank_LoeschenDialog(Medventory_DataTable);
        }

        private void Checkbox_Menge_CheckedChanged(object sender, EventArgs e)
        {
            if (Checkbox_Menge.Checked == true)
                Suche_Menge.Enabled = true;
            else
                Suche_Menge.Enabled = false;
        }

        private void Checkbox_Minimum_CheckedChanged(object sender, EventArgs e)
        {
            if (Checkbox_Minimum.Checked == true)
                Suche_Minimum.Enabled = true;
            else
                Suche_Minimum.Enabled = false;
        }

        private void Checkbox_Preis_CheckedChanged(object sender, EventArgs e)
        {
            if (Checkbox_Preis.Checked == true)
                Suche_Preis.Enabled = true;
            else
                Suche_Preis.Enabled = false;
        }

        private void Suche_Preis_ValueChanged(object sender, EventArgs e)
        {
            //Rundung
            decimal PreisGerundet = Suche_Preis.Value;
            Suche_Preis.Value = Math.Round(PreisGerundet, 2);
        }

        private void Button_Kostenrechnung_Click(object sender, EventArgs e)
        {
            //Kostenrechnung ausgeben

            // SQL Server - Verbindung
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            // SQL Query
            string SQL_Query = $"SELECT Art as Art, printf('%.2f', SUM(Preis*Menge)) AS Gesamtkosten FROM {Datenbank_Konfiguration.Table} GROUP BY Art UNION ALL SELECT 'Gesamt:' AS Art, printf('%.2f', SUM(Preis*Menge)) AS Gesamtkosten FROM {Datenbank_Konfiguration.Table}";

            // Daten in Datentabelle bertragen
            try
            {
                //Verbindung
                using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
                {
                    connection.Open();

                    SQLiteDataAdapter adapter = new SQLiteDataAdapter(SQL_Query, connection);

                    DataTable dataTable = new DataTable();

                    adapter.Fill(dataTable);

                    Medventory_DataTable.DataSource = dataTable;
                }
            }
            catch (SQLiteException Fehler_SQL)
            {
                // FEHLER: SQL
                string Fehler_Text = "SQL Fehler: " + Fehler_SQL.Message;
                //"\nError Code: " + Fehler_SQL.ErrorCode +
                //"\nState: " + Fehler_SQL.State +
                //"\nStack Trace: " + Fehler_SQL.StackTrace;

                MessageBox.Show(Fehler_Text, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);

            }
            catch (Exception)
            {
                // FEHLER: Sonstiges
                MessageBox.Show("Unbekannter Fehler.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Anzeige Datensatz anpassen
            Anzeige_Datensatz.Text = "Alle Eintrge";
            Anzeige_Datensatz.BackColor = Color.Gray;
            Button_Kostenrechnung.BackColor = Color.Orange;

            //Datum BIS aktualisieren
            Suche_DatumBis.Value = DateTime.Now;

            //Bearbeitungs-Buttons deaktivieren
            Button_Speichern.Enabled = false;
            Button_Loeschen.Enabled = false;

            //Warnungen formatieren
            if (Button_Warnungen.Text != "0 Warnungen")
                Button_Warnungen.BackColor = Color.DarkRed;
            else
                Button_Warnungen.BackColor = Color.Gray;
        }

        private void Medventory_DataTable_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            // Fehler bei Eingabe von falschem Variablen-Typ
            MessageBox.Show(
                "Das eingegebene Format fr das Feld ist falsch.",
                "FEHLER: Format",
                MessageBoxButtons.OK,
                MessageBoxIcon.Warning
            );
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //Warnungen anzeigen

            // SQL Server - Verbindung
            string SQL_Verbindung = $@"Data Source={Datenbank_Konfiguration.Server};Version=3;";

            // SQL Query
            string SQL_Query = $"SELECT * FROM {Datenbank_Konfiguration.Table} WHERE Menge < Minimum";

            // Daten in Datentabelle bertragen
            try
            {
                //Verbindung
                using (SQLiteConnection connection = new SQLiteConnection(SQL_Verbindung))
                {
                    connection.Open();

                    SQLiteDataAdapter adapter = new SQLiteDataAdapter(SQL_Query, connection);

                    DataTable dataTable = new DataTable();

                    adapter.Fill(dataTable);

                    Medventory_DataTable.DataSource = dataTable;

                    //Spalten-Reihenfolge festlegen
                    Medventory_DataTable.AutoGenerateColumns = false;
                    Medventory_DataTable.Columns["ID"].DisplayIndex = 0;
                    Medventory_DataTable.Columns["Artikel"].DisplayIndex = 1;
                    Medventory_DataTable.Columns["Art"].DisplayIndex = 2;
                    Medventory_DataTable.Columns["Hersteller"].DisplayIndex = 3;
                    Medventory_DataTable.Columns["Menge"].DisplayIndex = 4;
                    Medventory_DataTable.Columns["Minimum"].DisplayIndex = 5;
                    Medventory_DataTable.Columns["Preis"].DisplayIndex = 6;
                    Medventory_DataTable.Columns["Identifikationsnummer"].DisplayIndex = 7;
                    Medventory_DataTable.Columns["Aktualisiert"].DisplayIndex = 8;
                    Medventory_DataTable.AutoGenerateColumns = true;
                }
            }
            catch (SQLiteException Fehler_SQL)
            {
                // FEHLER: SQL
                string Fehler_Text = "SQL Fehler: " + Fehler_SQL.Message;
                //"\nError Code: " + Fehler_SQL.ErrorCode +
                //"\nState: " + Fehler_SQL.State +
                //"\nStack Trace: " + Fehler_SQL.StackTrace;

                MessageBox.Show(Fehler_Text, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);

            }
            catch (Exception)
            {
                // FEHLER: Sonstiges
                MessageBox.Show("Unbekannter Fehler.", "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //Anzeige Datensatz anpassen
            Anzeige_Datensatz.Text = "Alle Eintrge";
            Anzeige_Datensatz.BackColor = Color.Gray;
            Button_Warnungen.BackColor = Color.Red;

            //Datum BIS aktualisieren
            Suche_DatumBis.Value = DateTime.Now;

            //Bearbeitungs-Buttons deaktivieren
            Button_Speichern.Enabled = false;
            Button_Loeschen.Enabled = false;

            //Datum Format anpassen
            Medventory_DataTable.Columns["Aktualisiert"].DefaultCellStyle.Format = "yyyy-MM-dd HH:mm:ss";

            //Kostenrechnung Button anpassen
            Button_Kostenrechnung.BackColor = Color.Gray;
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {

        }

        private void Medventory_DataTable_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {

        }

        private void Medventory_DataTable_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            //[WARNUNGS-FORMATIERUNGEN] Wenn Werte "Menge" und "Minimum" enthalten sind...

            if (Medventory_DataTable.Columns.Contains("Menge") && Medventory_DataTable.Columns.Contains("Minimum"))
            {
                //Zellen-Formatierung bei Warnungen
                foreach (DataGridViewRow row in Medventory_DataTable.Rows)
                {
                    // Werte konvertieren
                    if (int.TryParse(row.Cells["Menge"].Value.ToString(), out int menge) &&
                        int.TryParse(row.Cells["Minimum"].Value.ToString(), out int minimum))

                        // Warnung formatieren wenn Menge < Minimum
                    if (menge < minimum)
                    {
                        row.DefaultCellStyle.BackColor = Color.Red;
                        row.DefaultCellStyle.ForeColor = Color.White;
                    }
                    else
                    {
                            row.DefaultCellStyle.BackColor = Color.White;
                            row.DefaultCellStyle.ForeColor = Color.Black;
                        }
                }
            }
        }
    }
}
